#set( $symbol_pound = '#' )
#set( $symbol_dollar = '$' )
#set( $symbol_escape = '\' )
package ${package}.${artifactId};

import java.util.List;
import javax.annotation.concurrent.ThreadSafe;
import javax.ws.rs.core.*;
import static javax.ws.rs.core.MediaType.*;
import javax.ws.rs.core.Response.ResponseBuilder;
import javax.ws.rs.ext.*;
import javax.xml.bind.JAXBElement;
import javax.xml.namespace.QName;
import net.java.truelicense.core.*;
import net.java.truelicense.obfuscate.Obfuscate;

/**
 * Maps a general security exception to a response.
 *
 * @author Christian Schlichtherle
 */
@Provider
@ThreadSafe
public final class LicenseExceptionMapper
implements ExceptionMapper<LicenseManagementException> {

    @Obfuscate private static final String NO_VALID_LICENSE_KEY_INSTALLED =
            "No valid license key installed.";

    @Obfuscate private static final String MESSAGE = "message";

    private static final QName message = new QName(MESSAGE);

    private @Context HttpHeaders headers;

    /**
     * Maps the given exception to a 402 "Payment required" status code.
     *
     * @see <a href="http://www.w3.org/Protocols/rfc2616/rfc2616-sec10.html"> http://www.w3.org/Protocols/rfc2616/rfc2616-sec10.html</a>
     */
    @Override public Response toResponse(final LicenseManagementException ex) {
        final String msg = ex instanceof LicenseValidationException
                ? ex.getMessage()
                : NO_VALID_LICENSE_KEY_INSTALLED;
        final List<MediaType> amt = headers.getAcceptableMediaTypes();
        final ResponseBuilder rb = Response.status(402);
        if (amt.contains(APPLICATION_JSON_TYPE))
            rb.type(APPLICATION_JSON_TYPE).entity('"' + msg + '"');
        else if (amt.contains(APPLICATION_XML_TYPE))
            rb.type(APPLICATION_XML_TYPE)
              .entity(new JAXBElement<String>(message, String.class, msg));
        else if (amt.contains(TEXT_XML_TYPE))
            rb.type(TEXT_XML_TYPE)
              .entity(new JAXBElement<String>(message, String.class, msg));
        else
            rb.type(TEXT_PLAIN_TYPE).entity(msg);
        return rb.build();
    }
}
